import { Component, EventEmitter, Input, Output } from '@angular/core';
import { UtilityService } from '../core/services/utility.service';
import { SessionStorageService } from '../core/services/session-storage.service';

const THEME_1 = {
    bgClass: 'header-bg-blue',
    iconColorClass: 'icon-color-white'
};

@Component({
    selector: 'collab-comp-header',
    templateUrl: './header.component.html',
    styleUrls: ['./header.component.scss']
})
export class HeaderComponent {
    @Input() logoName: string = '';
    @Input() menuList = [];
    @Input() headerTheme: string = '';

    @Output() public onLogOut: EventEmitter<any> = new EventEmitter<any>();
    @Output() public onContactUsClick: EventEmitter<any> = new EventEmitter<any>();

    name: string | null = '';
    role: string | null = '';
    constructor(private utilitySvc: UtilityService,
        private sessionStorageService: SessionStorageService) {
        this.utilitySvc.userProfileDetails$.subscribe((data) => {
            this.storeUserProfileDetails(data);
        })
    }

    public ngOnInit() {
        if(!!this.sessionStorageService.getStorage('name')) {
            this.name = this.sessionStorageService.getStorage('name');
        }
        if(!!this.sessionStorageService.getStorage('role')) {
            this.role = this.sessionStorageService.getStorage('role');
        }
    }

    public logout() {
        this.onLogOut.emit();
    }

    public storeUserProfileDetails(userDetails: any): void {
        if(!!userDetails.firstName && !!userDetails.lastName) {
            this.name = userDetails.firstName + ' ' + userDetails.lastName;
            this.sessionStorageService.setStorage('name', this.name);
        }
        if(!!userDetails.role) {
            this.role = userDetails.role;
            this.sessionStorageService.setStorage('role', this.role);
        }
    }

    public getHeaderBgClass(): string {
        return this.headerTheme === 'theme1' ? THEME_1.bgClass : '';
    }

    public getIconColorClass(): string{
        return this.headerTheme === 'theme1' ? THEME_1.iconColorClass : '';
    }

    public clickItem(menu: any) {
        switch (menu.value) {
            case 'Contact Us': this.onContactUsClick.emit();
        }
    }
}
