import { Component, ElementRef, EventEmitter, Input, OnDestroy, OnInit, Output } from '@angular/core';
import { ModalConfig } from './modal.config';
import { ModalService } from './service/modal.service';
import {ButtonConfig} from '../button/button-config.model';


@Component({
    selector: 'collab-comp-modal',
    templateUrl: './modal.component.html',
    styleUrls: ['./modal.component.scss']
})
export class ModalComponent implements OnInit, OnDestroy {
    private modalConfigInternal!: ModalConfig;
    @Input() public set modalConfig(val: ModalConfig){
        this.modalConfigInternal = val;
        this.setButtonConfigs(val);
    }
    public get modalConfig(): ModalConfig{
        return this.modalConfigInternal;
    }
    @Input() public id!: string;
    @Output() public onSubmitted = new EventEmitter<void>();
    @Output() public onClose = new EventEmitter<void>();
    @Input() public isFormValid!: boolean;

    private element: any;

    public buttonConfigClose!: ButtonConfig;

    public buttonConfigSubmit!: ButtonConfig;

    constructor(private modalService: ModalService, private el: ElementRef) {
        this.element = el.nativeElement;
    }

    public ngOnInit(): void {
        const modal = this;

        // ensure id attribute exists
        if (!this.id) {
            console.error('modal must have an id');
            return;
        }

        // move element to bottom of page (just before </body>) so it can be displayed above everything else
        document.body.appendChild(this.element);

        // close modal on background click
        this.element.addEventListener('click', function (e: any) {
            if (e.target.className === 'modal-background') {
                modal.onCancel();
            }
        });

        // add self (this modal instance) to the modal service so it's accessible from controllers
        this.modalService.add(this);
    }

    setButtonConfigs(val: ModalConfig){
        this.buttonConfigClose = {
            text: val?.closeButtonLabel !== undefined ? val?.closeButtonLabel : '',
            buttonType: 'secondary',
            imageUrl: '',
            linkURL: ''
        };
        this.buttonConfigSubmit = {
            text: val?.submitButtonLabel !== undefined ? val?.submitButtonLabel : '',
            buttonType: 'primary',
            imageUrl: '',
            linkURL: ''
        };
    }

    // remove self from modal service when directive is destroyed
    public ngOnDestroy(): void {
        this.modalService.remove(this.id);
        this.element.remove();
        document.body.classList.remove('modal-open');
    }

    // open modal
    public open(): void {
        this.element.style.display = 'block';
        document.body.classList.add('modal-open');
    }

    // close modal
    public close(): void {
        this.element.style.display = 'none';
        document.body.classList.remove('modal-open');
    }

    public onCancel(): void {
        this.close();
        this.onClose.emit();
    }

    public onSubmit(): void {
        if (this.isFormValid) {
            this.close();
        }
        this.onSubmitted.emit();
    }
}
