import { Component, Input } from '@angular/core';

@Component({
    selector: 'collab-comp-progress-bar-circle-wrapper',
    template: `
        <div class="progress-wrapper">
            <div class="current" [ngStyle]="getOverlayStyle()">
                {{ percentage }}
            </div>
            <collab-comp-round-progress
                    [current]="percentageParsed"
                    [max]="max"
                    [color]="gradient ? 'url(#gradient)' : color"
                    [background]="background"
                    [radius]="radius"
                    [stroke]="stroke"
                    [semicircle]="semicircle"
                    [rounded]="rounded"
                    [clockwise]="clockwise"
                    [responsive]="responsive"
                    [duration]="duration"
                    [animation]="animation"
                    [animationDelay]="animationDelay"
            ></collab-comp-round-progress>
        </div>
    `,
    styles: [`
        .progress-wrapper {
            position: relative;
            margin: 20px auto;
            font-size: 40px;
        }

        .current {
            position: absolute;
            color: black; /* #45ccce */
            font-weight: 100;
            line-height: 1;
        }

        round-progress {
            margin: auto;
        }
    `]
})

export class ProgressBarCircleWrapperComponent {
    @Input() public percentage: string = '';
    @Input() public color: string = '#0093C9';
    @Input() public background: string = '#E5E5E5';
    public max: number = 100;
    public stroke: number = 10;
    public radius: number = 40;
    public semicircle: boolean = false;
    public rounded: boolean = false;
    public responsive: boolean = false;
    public clockwise: boolean = true;
    public duration: number = 800;
    public animation: string = 'easeOutCubic';
    public animationDelay: number = 0;
    public animations: Array<string> = [
        'linearEase',
        'easeInQuad',
        'easeOutQuad',
        'easeInOutQuad',
        'easeInCubic',
        'easeOutCubic',
        'easeInOutCubic',
        'easeInQuart',
        'easeOutQuart',
        'easeInOutQuart',
        'easeInQuint',
        'easeOutQuint',
        'easeInOutQuint',
        'easeInSine',
        'easeOutSine',
        'easeInOutSine',
        'easeInExpo',
        'easeOutExpo',
        'easeInOutExpo',
        'easeInCirc',
        'easeOutCirc',
        'easeInOutCirc',
        'easeInElastic',
        'easeOutElastic',
        'easeInOutElastic',
        'easeInBack',
        'easeOutBack',
        'easeInOutBack',
        'easeInBounce',
        'easeOutBounce',
        'easeInOutBounce',
    ];
    public gradient: boolean = false;
    public realCurrent: number = 0;

    public get percentageParsed(){
        return parseFloat(this.percentage);
    }
    public getOverlayStyle() {
        const isSemi = this.semicircle;
        const transform = (isSemi ? '' : 'translateY(-50%) ') + 'translateX(-50%)';

        return {
            top: isSemi ? 'auto' : '50%',
            bottom: isSemi ? '5%' : 'auto',
            left: '50%',
            transform,
            'font-size': this.radius / 1.8 + 'px',
            'font-weight': 'bold'
        };
    }
}
