import {ModuleWithProviders, NgModule} from '@angular/core';
import { HTTP_INTERCEPTORS } from '@angular/common/http';
import {CommonModule} from '@angular/common';
import {NgIdleKeepaliveModule} from '@ng-idle/keepalive';
import {FormsModule, ReactiveFormsModule} from "@angular/forms";
import {AgGridModule} from 'ag-grid-angular';

import { HeaderComponent } from './header/header.component';
import { FooterComponent } from './footer/footer.component';
import { RequestInterceptor } from './core/services/request.interceptor.service';
import { CookieService } from './core/services/cookie.service';
import { SessionStorageService } from './core/services/session-storage.service';
import { AuthenticationService } from './core/services/authentication.service';
import { UtilityService } from './core/services/utility.service';
import { ModalService } from './modal/service/modal.service';
import { ModalComponent } from './modal/modal.component';
import { ImgGeneratorComponent } from './img-generator/img-generator.component';
import { ButtonComponent } from './button/button.component';
import { TimeoutComponent } from './timeout/timeout.component';
import { TimeoutService } from './timeout/timeout.service';
import { RoundProgressComponent } from './progress-bar/circular-progress/round-progress.component';
import { ProgressBarCircleWrapperComponent } from './progress-bar/circular-progress/progress-example/progress-bar-circle.wrapper.component';
import { ProgressBarHorizontalComponent } from './progress-bar/progress-bar-horizontal';
import { TooltipDirective } from './tooltip/tooltip.directive';
import { PolicyBannerComponent } from './policy-banner/policy-banner.component';
import {InputTextboxComponent} from "./form-component/input-textbox/input-textbox.component";
import {FormInlineErrorComponent} from "./form-inline-error/form-inline-error.component";
import {TableGridComponent} from "./table-grid/table-grid.component";
import {TableGridPaginationComponent} from "./table-grid/table-grid-pagination/table-grid-pagination.component";


@NgModule({
  imports: [
      CommonModule,
      NgIdleKeepaliveModule.forRoot(),
      FormsModule,
      ReactiveFormsModule,
      AgGridModule
  ],
  declarations: [
    FooterComponent,
    ImgGeneratorComponent,
    HeaderComponent,
    RoundProgressComponent,
    ProgressBarCircleWrapperComponent,
    ProgressBarHorizontalComponent,
    ButtonComponent,
    ModalComponent,
    TooltipDirective,
    PolicyBannerComponent,
    TimeoutComponent,
    InputTextboxComponent,
    FormInlineErrorComponent,
    TableGridComponent,
    TableGridPaginationComponent,
  ],
  exports: [
    FooterComponent,
    ImgGeneratorComponent,
    HeaderComponent,
    RoundProgressComponent,
    ProgressBarCircleWrapperComponent,
    ProgressBarHorizontalComponent,
    ButtonComponent,
    ModalComponent,
    TooltipDirective,
    PolicyBannerComponent,
    TimeoutComponent,
    InputTextboxComponent,
    FormInlineErrorComponent,
    TableGridComponent,
    TableGridPaginationComponent,
  ],
  providers: [{provide: HTTP_INTERCEPTORS, useClass: RequestInterceptor, multi: true},
    ModalService]
})
export class UiModule {
  static forRoot(): ModuleWithProviders<any> {
    return {
      ngModule: UiModule,
      providers: [ ModalService,
        CookieService,
        TimeoutService,
        SessionStorageService,
        AuthenticationService,
        UtilityService
      ]
    };
  }
}
